#!/bin/bash
# Peter Poeml poeml@suse.de

: {$KEYFILE:=/etc/named.keys}

# This is where the keys are created 
cd $ROOT/$(dirname $KEYFILE)

# determine the BIND version
if [ -f /usr/sbin/rndc ]; then 
	bind9=true
elif [ -f /usr/sbin/ndc ]; then
	bind9=false
else
	echo could not determine the BIND version. Exiting.
	exit 1
fi

umask 600

# generate a 512 bit HMAC-MD5 Zone (DNS validation) key
if $bind9; then 
	keyfile=$(/usr/sbin/dnssec-keygen -a hmac-md5 -b 512 -n user DHCP_UPDATER)
else
	keyfile=$(/usr/sbin/dnskeygen -H 512 -z -c -n DHCP_UPDATER)
	# dhskeygen has (had) a weekness, it puts one key into a world readable file
	# (see http://xforce.iss.net/alerts/advise78.php)
	chmod 600 $keyfile*
fi
# now we've got files like these:
# -rw-------    1 root     root           77 Sep 11 01:03 KDHCP_UPDATER.+157+00000.private
# -rw-r--r--    1 root     root           58 Sep 11 01:03 KDHCP_UPDATER.+157+00000.key
#
#                                                          ------------      -----
#                                                          name              key id
#
#                                                                        ---
#                                                                        157 is short
#                                                                        for hmac-md5
echo $keyfile

# read the secret 
while read line; do
	case $line in 
	Key:*)	secret=${line#* }
	esac
done < $keyfile.private


cat >$KEYFILE <<-EOF

# generated by $(basename $0) on $(date)

key DHCP_UPDATER {
	$(if $bind9; then 
		echo "algorithm hmac-md5;"
	else
		echo "algorithm HMAC-MD5.SIG-ALG.REG.INT;"
	fi)
        secret "$secret";
};


EOF

# set permissions
chown root.named $KEYFILE
chmod 640 $KEYFILE
